<?php

namespace FM\Controller;

use Teknisa\Libs\Controller\CRUD;
use Teknisa\Libs\Service\CRUD as CRUDService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Util\Utilities;

class FMCrud extends CRUD
{

    protected $dataSourceName;

    public function __construct(CRUDService $crudService, Environment $environment)
    {
        parent::__construct($crudService, $environment);
    }

    public function find(Request\Filter $request, Response $response)
    {
        try {
            $this->setDbNameParameter($request);
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 500));
        }
    }

    public function consertarIntervaloFixoNovoCodigo($request)
    {
        $rows = (method_exists($request, "getDataSet")) ? $request->getDataSet()->getRows() : array($request->getRow());
        foreach ($rows as &$row) {
            if (isset($row['__new_code_length']) && $row['__new_code_length'] == 5) {
                $row['__new_code_length'] = 1;
            }
        }
        unset($row);

        return $request;
    }

    public function save(Request $request, Response $response)
    {
        try {
            $this->setDbNameParameter($request);
            $this->dataSourceName = $request->getParameter(self::DATABASE_NAME);
            $this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $request = $this->consertarIntervaloFixoNovoCodigo($request);
            $rows    = $this->crudService->getRowsToSave($request, $this->dataSourceName);
            $this->crudService->save($this->dataSourceName, $rows);
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);

            $currentRow = current($request->getDataSet()->getRows());

            if (isset($currentRow['saveHistoryTable'])) {
                if (!$currentRow['__is_new']) {
                    if (!$this->checkHistory($response, $currentRow)) {
                        // is new history
                        $currentRow['__is_new'] = true;
                    }
                }

                $tableMasterPK = strtoupper($currentRow['saveHistoryTable']['__table_master_pk']);
                $newCode = array_filter($this->crudService->getColumnsToReturn(), function ($element) use ($tableMasterPK) {
                    return $element[$tableMasterPK];
                });

                $this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($currentRow['saveHistoryTable']['__table_history_name']);
                $this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
                $currentRow[strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])] = intval($newCode[0][strtoupper($currentRow['saveHistoryTable']['__table_master_pk'])]);

                // set history properties
                $currentRow['__new_code_pk'] = $currentRow['saveHistoryTable']['__table_history_name'];
                $currentRow['__table_pk'] = $currentRow['saveHistoryTable']['__table_history_pk'];
                $currentRow['__columns_to_return'] = $currentRow['saveHistoryTable']['__columns_to_return'];

                $newDataSet = new DataSet($this->dataSourceName, array($currentRow));
                $newRequest = new \Zeedhi\Framework\DTO\Request\DataSet(
                    $newDataSet,
                    $request->getMethod(),
                    $request->getRoutePath(),
                    $request->getUserId()
                );
                $rows = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
                $this->crudService->save($this->dataSourceName, $rows);
            }
            $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 500));
        }
    }

    public function delete(Request $request, Response $response)
    {
        try {
            $this->setDbNameParameter($request);
            $this->dataSourceName = $request->getParameter(self::DATABASE_NAME);
            //$this->dataSourceName = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $rows = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
            $dataSet = new DataSet($this->dataSourceName, (array)$rows);
            $this->crudService->delete($response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Retorna um novo objeto Request\Filter com o filtro de NRORG IN (0, logado)
     */
    public function getNovoFilterRequestComNrorgLogadoOuZero(Request $request)
    {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();

        $newFilterCriteria = new FilterCriteria($filterCriteria->getDataSourceName(), $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $this->crudService->keepOrderBy($filterCriteria, $newFilterCriteria);

        $newFilterCriteria->addCondition("NRORG", "IN", array(0, $this->environment->getNrOrgTrab()));

        return new Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    }

    protected function setDbNameParameter($request)
    {
        try {
            $dbname = $request->getParameter(self::DATABASE_NAME);
        } catch (\Exception $e) {
            $request->setParameter(self::DATABASE_NAME, $this->dataSourceName);
        }
    }
}
