<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00030000TarefasRecorrentesFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getDadosTarefaRecorrente()
    {
        $sql = "SELECT FCM_TAREFARECORRENTE.NRTAREFARECORRENTE, FCM_TAREFARECORRENTE.NMTAREFA, FCM_TAREFARECORRENTE.TPLOCALTAREFA, FCM_TAREFARECORRENTE.IDCRIARVISTORIA,
                        FCM_TAREFARECORRENTE.LOCALTAREFA, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_TAREFARECORRENTE.DIASSEMANA, 
                        FCM_TAREFARECORRENTE.HORARIO, FCM_TAREFARECORRENTE.DURACAO, TO_CHAR(FCM_TAREFARECORRENTE.DTHORAREINICIA, 'MM/DD/YYYY') AS DTHORAREINICIA, 
                        FCM_TAREFARECORRENTE.NRCHECKLIST, FCM_TAREFARECORRENTE.ITENSCHECKLIST, FCM_TAREFARECORRENTE.ITENSTAREFA, 
                        FCM_TAREFARECORRENTE.CRITERIOS, FCM_TAREFARECORRENTE.CDOPERTAREFA, FUNCTAREFA.NMOPERADOR, 
                        FCM_TAREFARECORRENTE.CDOPERVISTORIA, FUNCVISTORIA.NMOPERADOR AS SUPERVISOR, FCM_LOCALCLI.CDCLIENTE, 
                        CLIENTE.NMRAZSOCCLIE, FUNCTAREFA.CDOPERADOR, FCM_LOCALCLI.NRLOCALCLIENTE,FCT_CONTRATO.NRCONTRATOFCT, FCM_TAREFARECORRENTE.NRSEQCONTRATO, FCM_TAREFARECORRENTE.IDERRO,
                        CASE FCM_TAREFARECORRENTE.IDSITUACAO 
                        WHEN '1' THEN 'Em Andamento'
                        WHEN '2' THEN 'Pausada'
                        WHEN '3' THEN 'Cancelada'
                        WHEN '4' THEN 'Erro na Geração' END AS NMIDSITUACAO
            FROM FCM_TAREFARECORRENTE
            LEFT JOIN OPERADOR FUNCTAREFA
            ON FCM_TAREFARECORRENTE.CDOPERTAREFA = FUNCTAREFA.CDOPERADOR
            LEFT JOIN OPERADOR FUNCVISTORIA
            ON FCM_TAREFARECORRENTE.CDOPERVISTORIA = FUNCVISTORIA.CDOPERADOR
            LEFT JOIN FCM_LOCALCLI
            ON FCM_TAREFARECORRENTE.LOCALTAREFA = FCM_LOCALCLI.NRLOCALCLIENTE
            LEFT JOIN FCM_TIPOLOCALCLI
            ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
            LEFT JOIN CLIENTE
            ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
            LEFT JOIN FCM_CHECKLIST
            ON FCM_TAREFARECORRENTE.NRCHECKLIST = FCM_CHECKLIST.NRCHECKLIST
            LEFT JOIN FCT_CONTRATO
            ON FCT_CONTRATO.NRSEQCONTRATO = FCM_TAREFARECORRENTE.NRSEQCONTRATO ";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertTarefasRecorrentes($param)
    {
        $sql = "INSERT INTO FCM_TAREFARECORRENTE ( 
                                NRTAREFARECORRENTE, NMTAREFA, TPLOCALTAREFA, LOCALTAREFA, 
                                NRCHECKLIST, ITENSCHECKLIST, ESCOPOSTAREFA, ITENSTAREFA, 
                                CRITERIOS, DIASSEMANA, HORARIO, DURACAO, 
                                IDCRIARVISTORIA, NRORG, IDATIVO, DTINCLUSAO, 
                                CDOPERINCLUSAO, NRORGINCLUSAO, IDSITUACAO, CDOPERTAREFA, 
                                CDOPERVISTORIA, CDCLIENTE, NRSEQCONTRATO)
                     VALUES (   :NRTAREFARECORRENTE, :NMTAREFA, :TPLOCALTAREFA, :LOCALTAREFA, 
                                :NRCHECKLIST, :ITENSCHECKLIST, :ESCOPOSTAREFA, :ITENSTAREFA, 
                                :CRITERIOS, :DIASSEMANA, :HORARIO, :DURACAO, 
                                :IDCRIARVISTORIA, :NRORG, :IDATIVO, SYSDATE, 
                                :CDOPERINCLUSAO, :NRORGINCLUSAO, :IDSITUACAO, :CDOPERTAREFA, 
                                :CDOPERVISTORIA, :CDCLIENTE, :NRSEQCONTRATO)";

        $this->connection->executeStatement($sql, $param);
    }

    public function mudarStatusTarefaRecorrente($param)
    {

        $sql = "UPDATE FCM_TAREFARECORRENTE
                   SET IDSITUACAO = {$param['IDSITUACAO']},
                       DTULTATU = SYSDATE
                 WHERE NRTAREFARECORRENTE IN ({$param['NRTAREFARECORRENTE']})";

        $this->connection->executeStatement($sql);
    }

    public function editarTarefaRecorrente($param)
    {
        $sql = "UPDATE FCM_TAREFARECORRENTE
                   SET DIASSEMANA = :DIASSEMANA,
                       HORARIO = :HORARIO,
                       DURACAO = :DURACAO,
                       CDOPERTAREFA = :CDOPERTAREFA,
                       CDOPERVISTORIA = :CDOPERVISTORIA,
                       CDOPERULTATU = :CDOPERULTATU,
                       DTULTATU = SYSDATE
                 WHERE NRTAREFARECORRENTE = :NRTAREFARECORRENTE";

        $this->connection->executeStatement($sql, $param);
    }

    public function retirarReinicio($retiraReinicio)
    {
        $sql = "UPDATE FCM_TAREFARECORRENTE
                   SET DTHORAREINICIA = :DTHORAREINICIA
                 WHERE NRTAREFARECORRENTE = :NRTAREFARECORRENTE";

        $this->connection->executeStatement($sql, $retiraReinicio);
    }

    public function deletarPeriodoTarefaRecorrente($params, $filter)
    {
        $sql = "SELECT (LISTAGG(NRCHECKLISTAGEND, ',')  WITHIN GROUP (ORDER BY NRCHECKLISTAGEND)) AS NRCHECKLISTAGEND
                                                                         FROM FCM_TAREFAAGEND
                                                                         WHERE NRTAREFARECORRENTE IN (" . $params . ") " . $filter;
        
        $RETORNO = $this->connection->executeQuery($sql)->fetchAllAssociative();

        if ($RETORNO[0]['NRCHECKLISTAGEND'] != null) {
            $param = array(
                "NRCHECKLISTAGEND" => $RETORNO[0]['NRCHECKLISTAGEND']
            );

            $sql = "UPDATE FCM_CHECKLISTAGEND  
                       SET IDATIVO = 'N'
                     WHERE NRCHECKLISTAGEND IN ({$param['NRCHECKLISTAGEND']})";

            $this->connection->executeStatement($sql);

            $sql = "UPDATE FCM_GERENCIAMENTOCHK 
                       SET IDATIVO = 'N'
                     WHERE NRCHECKLISTAGEND IN ({$param['NRCHECKLISTAGEND']})";

            $this->connection->executeStatement($sql);
        }
    }

    public function pegarTodosDados($params)
    {
        $sql = "SELECT * FROM FCM_TAREFARECORRENTE
                WHERE NRTAREFARECORRENTE = $params";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getTarefasRecorrente()
    {
        $sql = "SELECT * FROM FCM_TAREFARECORRENTE";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function ativaTarefaRecorrente($params, $filter)
    {
        $sql = "SELECT (LISTAGG(NRCHECKLISTAGEND, ',')  WITHIN GROUP (ORDER BY NRCHECKLISTAGEND)) AS NRCHECKLISTAGEND
                                                                         FROM FCM_TAREFAAGEND
                                                                         WHERE NRTAREFARECORRENTE = " . $params . " " . $filter;

        $RETORNO = $this->connection->executeQuery($sql)->fetchAllAssociative();

        $sql = "UPDATE FCM_CHECKLISTAGEND  
                       SET IDATIVO = 'S'
                     WHERE NRCHECKLISTAGEND IN ({$RETORNO[0]['NRCHECKLISTAGEND']})";

        $this->connection->executeStatement($sql);

        $sql = "UPDATE FCM_GERENCIAMENTOCHK 
                       SET IDATIVO = 'S'
                     WHERE NRCHECKLISTAGEND IN ({$RETORNO[0]['NRCHECKLISTAGEND']})";

        $this->connection->executeStatement($sql);
    }

    public function validaPeriodo($param)
    {
        $sql =  "SELECT COUNT(*) AS QTD FROM FCM_TAREFAAGEND WHERE NRTAREFARECORRENTE = {$param}";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getErroRelatorio()
    {
        $sql =  "SELECT DISTINCT T1.NRTAREFARECORRENTE, T2.NRTAREFARECORRENTE as VALIDA
                 FROM FCM_TAREFARECORRENTE T1
                LEFT JOIN FCM_ERROSTAREFAREC T2 
                ON T1.NRTAREFARECORRENTE = T2.NRTAREFARECORRENTE";
        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function setIdErro($iderro, $nrrecorrente)
    {

        $sql = "UPDATE FCM_TAREFARECORRENTE
                SET IDERRO = '$iderro'
                WHERE NRTAREFARECORRENTE = $nrrecorrente";
        $this->connection->executeStatement($sql);
    }

    public function dataRetorno($dataReinicio, $NR)
    {
        $sql = "UPDATE FCM_TAREFARECORRENTE
                   SET DTHORAREINICIA = TO_DATE('{$dataReinicio}','DD/MM/YYYY')
                   WHERE NRTAREFARECORRENTE = $NR ";

        $this->connection->executeStatement($sql);
    }

    public function getAllTasks()
    {
        $sql = "SELECT NRCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST, NRFUNCIONARIO, 
                       IDSITUACAOAGEND, TO_CHAR(DTHORAAGEND,'MM/DD/YYYY HH24:MI') AS DTHORAAGEND, IDATIVO, NRORG, 
                       DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, 
                       CDOPERULTATU, NRORGULTATU, NMCHECKLISTAGEND, 
                       DSJUSTCANCELAMENTO, TO_CHAR(DTHRFINALAGEND, 'MM/DD/YYYY HH24:MI') AS DTHRFINALAGEND, IDNOTIFICADO, CDOPERADOR, 
                       IDOBRIGAOBSERVACAO, NRSEQCONTRATO FROM FCM_CHECKLISTAGEND WHERE NRCHECKLISTAGEND IN 
                                                                            (SELECT NRCHECKLISTAGEND FROM FCM_TAREFAAGEND WHERE IDATIVO = 'S')";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getDuracao($params)
    {
        $sql = "SELECT DURACAO , HORARIO, DIASSEMANA, CDOPERTAREFA, CDOPERVISTORIA, NMTAREFA, NRTAREFARECORRENTE
                  FROM FCM_TAREFARECORRENTE
                 WHERE NRTAREFARECORRENTE = $params ";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function criaRelatorio($param)
    {
        $sql = "INSERT INTO FCM_ERROSTAREFAREC (NRERROTAREFAREC,   NRTAREFARECORRENTE, NMTAREFARECORRENTE,
                                                CDOPERRESPONSAVEL, CDOPERENCARREGADO,  DTINICIAL,
                                                DTFINAL,           DURACAO,            NRORG,
                                                IDATIVO,           DTINCLUSAO,         NRORGINCLUSAO)
                     VALUES ( 
                        :NRERROTAREFAREC,   :NRTAREFARECORRENTE, :NMTAREFARECORRENTE,
                        :CDOPERRESPONSAVEL, :CDOPERENCARREGADO,  :DTINICIAL,
                        :DTFINAL,           :DURACAO,            :NRORG,
                        :IDATIVO,           SYSDATE,             :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $param);
    }

    public function getRelatoriosConflitos($filter)
    {
        $sql = "SELECT REL.NRERROTAREFAREC, REL.NRTAREFARECORRENTE, REL.NMTAREFARECORRENTE,
                       REL.DTINICIAL AS DHINICIALNOVA, REL.DTFINAL AS DHFINALNOVA, OPER.NMOPERADOR, REL.DURACAO, REL.CDOPERRESPONSAVEL AS CDOPERADOR,
                       TO_CHAR(REL.DTINICIAL, 'HH24:MI') AS HORAINICIO
                 FROM FCM_ERROSTAREFAREC REL
                 JOIN OPERADOR OPER
                   ON REL.CDOPERRESPONSAVEL = OPER.CDOPERADOR $filter";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function delRegistroRelatorio($param)
    {

        $sql = "DELETE FCM_ERROSTAREFAREC
                WHERE NRERROTAREFAREC IN ($param)";

        $this->connection->executeStatement($sql);
    }

    public function excluiRegistroRelatorio($params)
    {
        $sql = "DELETE FCM_ERROSTAREFAREC WHERE NRERROTAREFAREC IN ($params)";

        $this->connection->executeStatement($sql);
    }
}
