/* eslint-disable */
import {
  Http, IDictionary, Loader, Metadata,
} from '@zeedhi/core';
import {
  AlertService, DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class TabCommunicationController {
  private addCommunication!: Modal;

  private editionCommunication!: Modal;

  public BPController = Loader.getInstance('BusinessPartnerController');

  public setFilterGrid() {
    const NRPARCNEGOCIO = window.location.search.split('?')[1];
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabCommunication');
    grid.datasource.setFilter({
      NRPARCNEGOCIO: NRPARCNEGOCIO,
    });
    grid.datasource.get();
  }

  public onMountNameModal() {
    Metadata.getInstance<Text>('nameOfCorporate-addCommunication').text = this.BPController.NAMEOFCORPORATE;
  }

  public async openModalCommunication() {
    this.addCommunication = await ModalService.createFromJson('BusinessPartner/Modal/modal-add-communication', true);
    this.addCommunication.show();
  }

  public closeModalCommunication() {
    this.addCommunication.hide();
    this.addCommunication?.destroy();
  }

  public async openModalEdition({ row }: any) {
    this.editionCommunication = await ModalService.createFromJson('BusinessPartner/Modal/modal-edition-communication', true);
    this.editionCommunication.show();
    await this.mountInformationOfCommunication(row.NRCOMUNICAPARC);
  }

  public closeModalEdition() {
    this.editionCommunication.hide();
    this.editionCommunication?.destroy();
  }

  public async mountInformationOfCommunication(NRCOMUNICAPARC: number) {
    await Http.post('/getSpecificWayOfCommunication', { NRCOMUNICAPARC }).then((response: IDictionary) => {
      response.data.dataset.communicFound.forEach((data: IDictionary) => {
        Metadata.getInstance<Text>('nameOfCorporate-editionCommunication').text = this.BPController.NAMEOFCORPORATE;
        Metadata.getInstance<Text>('NRCOMUNICAPARC').text = NRCOMUNICAPARC.toString();
        Metadata.getInstance<Select>('formCommunication-formOfCommunication').setValue(data.CDFORMACOMU);
        Metadata.getInstance<TextInput>('formCommunication-Prefix').value = data.CDPREFIXCOMUPARC;
        Metadata.getInstance<TextInput>('formCommunication-Description').value = data.CDCOMUNICAPARC;
      });
    });
  }

  public insertNewCommunicationData() {
    const formCommunication = Metadata.getInstance<Form>('form-addCommunication');
    const prefix = Metadata.getInstance<TextInput>('formPrefix').value;
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabCommunication');

    if (formCommunication.validate()) {
      LoadingService.show('Adicionando Forma de Comunicação...');
      Http.post('/newWayOfCommunication', {
        NRPARCNEGOCIO: window.location.search.split('?')[1],
        CDFORMACOMU: Metadata.getInstance<Select>('formOfCommunication').value,
        CDPREFIXCOMUPARC: (prefix === '') ? null : prefix,
        CDCOMUNICAPARC: Metadata.getInstance<TextInput>('formDescription').value,
        CDOPERINCLUSAO: localStorage.getItem('CDOPERADOR'),
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.communic[0];
        if (returnMessage === 'OK') {
          this.alertMessages('Forma de Comunicação adicionada com sucesso!', 'success');
          this.closeModalCommunication();
          grid.datasource.get();
        } else {
          this.closeModalCommunication();
          DialogService.show({
            name: 'warning-dialog',
            type: 'warning',
            title: 'Erro!',
            text: 'Operação bloqueada! Esta forma de comunicação não pôde ser adicionada.<br><br>Se o problema persistir entre em contato com o suporte.',
          });
        }
      });
      LoadingService.hide();
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

  public updateCommunication() {
    const formEditCommunication = Metadata.getInstance<Form>('formCommunication');
    const prefix = Metadata.getInstance<TextInput>('formCommunication-Prefix').value;
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabCommunication');

    if (formEditCommunication.validate()) {
      LoadingService.show('Salvando alterações...');
      Http.post('/updateWayOfCommunication', {
        NRCOMUNICAPARC: Metadata.getInstance<Text>('NRCOMUNICAPARC').text,
        NRPARCNEGOCIO: window.location.search.split('?')[1],
        CDFORMACOMU: Metadata.getInstance<Select>('formCommunication-formOfCommunication').value,
        CDPREFIXCOMUPARC: (prefix === '') ? null : prefix,
        CDCOMUNICAPARC: Metadata.getInstance<TextInput>('formCommunication-Description').value,
        CDOPERULTATU: localStorage.getItem('CDOPERADOR'),
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.communic[0];
        if (returnMessage === 'OK') {
          this.alertMessages('Forma de Comunicação alterada com sucesso!', 'success');
          this.closeModalEdition();
          grid.datasource.get();
        } else {
          this.closeModalEdition();
          DialogService.show({
            name: 'warning-dialog',
            type: 'warning',
            title: 'Erro!',
            text: 'Operação bloqueada! Esta forma de comunicação não pôde ser alterada.<br><br>Se o problema persistir entre em contato com o suporte.',
          });
        }
      });
      LoadingService.hide();
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

  public showDeleteDialog({ event } : any) {
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabCommunication');
    event.preventDefault();

    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Excluir forma de Comunicação',
      text: 'Todas as informações serão perdidas se concluir está ação. Deseja continuar?',
      buttons: [
        {
          name: 'button-no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => DialogService.hide(),
          },
        },
        {
          name: 'button-yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: '{{TabCommunicationController.deleteWayOfCommunication}}'
          },
          outline: true,
        },
      ],
    });
  }

  public async deleteWayOfCommunication() {
    LoadingService.show('Deletando...');
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabCommunication');
    const NRS = (grid.selectedRows.map((element: any) => element.NRCOMUNICAPARC)).toString();

    await Http.post('/deleteWayOfCommunication', { NRCOMUNICAPARC: NRS }).then((response: IDictionary) => {
      if (response.data.dataset.communic[0] === 'OK') {
        this.alertMessages('Forma de Comunicação excluída com sucesso', 'success');
        grid.datasource.get();
        grid.selectedRows = [];
        DialogService.hide();
      } else {
        DialogService.show({
          name: 'warning-dialog',
          type: 'warning',
          title: 'Erro!',
          text: 'Operação bloqueada! Esta forma de comunicação não pôde ser excluída.<br><br>Se o problema persistir entre em contato com o suporte.',
        });
      }
    });
    LoadingService.hide();
  }

  public alertMessages(message: string, type: string) {
    AlertService.show({
      name: 'basic-alert',
      text: message,
      top: true,
      left: false,
      timeout: 3000,
      color: type,
    });
  }
}
