/* eslint-disable */
import {
  Metadata, Http, IDictionary,
} from '@zeedhi/core';
import {
  Text, Grid, Progress, ApexChart, Select,
} from '@zeedhi/common';

export class Fm00000000DashboardController {
  // função para pegar data/hora atual para calculos de atrasado/finalizado e concluido
  public getDataHoraAtual() {
    const horaAtual = new Date();
    const ano = horaAtual.getFullYear();
    const mes = horaAtual.getMonth();
    const dia = horaAtual.getDate();
    const hora = horaAtual.getHours();
    const minutos = horaAtual.getMinutes();
    const segundos = horaAtual.getSeconds();
    const dataHoraAtual = `${dia}-${mes + 1}-${ano} ${hora}:${minutos}:${segundos}`;
    return dataHoraAtual;
  }

  public async preencheProgressoDiario() {
    Http.get('/getDadosProgress').then((response) => {
      const data = response.data.dataset.dados.flat(3);
      Metadata.getInstance<Text>('tituloProgress').text = `Tarefas Concluidas: ${data[0].QTDTAREFASCONCLUIDAS}/${data[1].QTDTAREFAS}`;
      const percentual = Math.floor((data[0].QTDTAREFASCONCLUIDAS * 100) / data[1].QTDTAREFAS);
      Metadata.getInstance<Progress>('slotProgress').value = `${percentual}%`;
      Metadata.getInstance<Text>('percentageText').text = `${percentual}%`;
    });
  }

  // ---------------- FUNCIONALIDADES DA TELA DASH GERENCIAL --------------------------

  private dataFilter: IDictionary = {
    selectUnit: '',
    selectService: '',
    selectLocal: '',
  };

  public filter() {
    this.dataFilter.selectUnit = this.getUnit;
    this.dataFilter.selectService = this.getService;
    this.dataFilter.selectLocal = this.getLocal;
    this.dataBaseChartPieUnit();
    this.dataBaseChartPieClean();
    this.dataBaseChartBarUnconformities();
    this.dataBaseChartBarAvgTime();
  }

  get getUnit() { // pegar o select Local
    return Metadata.getInstance<Select>('selectUnit').value || '';
  }

  get getService() { // pegar o select Situação/Status
    return Metadata.getInstance<Select>('selectService').value || '';
  }

  get getLocal() { // pegar o select Situação/Status
    return Metadata.getInstance<Select>('selectLocal').value || '';
  }

  public setUnit({ component }:{ component:Select }) {
    Http.post('/localsForChart', { id: 1 }).then((response: IDictionary) => {
      response.data.dataset.locals.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMLOCALCLIENTE,
          value: element.NRLOCALCLIENTE,
        });
      });
    });
  }

  public setLocal({ component }:{ component:Select }) {
    Http.post('/localsForChart', { id: 3 }).then((response: IDictionary) => {
      response.data.dataset.locals.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMLOCALCLIENTE,
          value: element.NRLOCALCLIENTE,
        });
      });
    });
  }

  public setActivities({ component }:{ component:Select }) {
    Http.get('/setActivities').then((response: IDictionary) => {
      response.data.dataset.act.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMATIVIDADE,
          value: element.NRATIVIDADE,
        });
      });
    });
  }
  
  // Gráfico de Tempo Médio
  public dataBaseChartBarAvgTime() {
    const component = Metadata.getInstance<ApexChart>('bar-chart-avg-time');
    const datab: any = [];
    const dataColors: any = [];

    Http.post('/dataBaseChartBarAvgTime', this.dataFilter).then((response: IDictionary) => {
      if (response.data.dataset.dbAvgTime.length) {
        response.data.dataset.dbAvgTime.forEach((dados: IDictionary) => {
          datab.push({
            name: dados.NMATIVIDADE,
            data: [dados.TEMPOMEDIOTAREFAS],
          });
          dataColors.push(dados.DSCORIDATIVIDADE);
        });
      } else {
        datab.push({
          name: 'Nenhum dado disponível',
          data: [100],
        });
        dataColors.push('#57737a');
      }
      const options = {colors: dataColors, series: datab};
      component.updateChart(options);
    });
  }

  public dataBaseChartBarUnconformities() {
    const component = Metadata.getInstance<ApexChart>('bar-chart-unconformities');
    const datab: any = [];
    const dataColors: any = [];

    Http.post('/dataBaseChartBarUnconformities', this.dataFilter).then((response: IDictionary) => {
      if (response.data.dataset.dbChartb.length) {
        response.data.dataset.dbChartb.forEach((dados: IDictionary) => {
          datab.push({
            name: dados.NMATIVIDADE,
            data: [dados.QUANTIDADE],
          });
          dataColors.push(dados.DSCORIDATIVIDADE);
        });
      } else {
        datab.push({
          name: 'Nenhum dado disponível',
          data: [100],
        });
        dataColors.push('#57737a');
      }
     const options = {colors: dataColors, series: datab};
     component.updateChart(options);
    });
  }

  // Gráfico de Tarefas Realizadas
  public dataBaseChartPieClean() {
    const component = Metadata.getInstance<ApexChart>('second-pie-chart-tasks');

    const dataLabels: any = [];
    const dataSeries: any = [];
    const dataColors: any = [];

    Http.post('/dataBaseChartPieClean', this.dataFilter).then((response: IDictionary) => {
      if (response.data.dataset.dbChartp.length) {
        response.data.dataset.dbChartp.forEach((dados: IDictionary) => {
          dataLabels.push(dados.NMATIVIDADE);
          dataSeries.push(dados.QUANTIDADE);
          dataColors.push(dados.DSCORIDATIVIDADE);
        });
      } else {
        dataLabels.push('Nenhum dado disponível');
        dataSeries.push(100);
        dataColors.push('#57737a');
      }
    });
    const options = {colors: dataColors, series: dataSeries, labels: dataLabels};
    component.updateChart(options);
  }

  //Gráfico de Tarefas por unidade
  public dataBaseChartPieUnit() {
    const component = Metadata.getInstance<ApexChart>('pie-chart-unit');

    const dataLabels: any = [];
    const dataSeries: any = [];
    const dataColors: any = [];

    Http.post('/dataBaseChartPieUnit', this.dataFilter).then((response: IDictionary) => {
      if (response.data.dataset.dbChartpu.length) {
        response.data.dataset.dbChartpu.forEach((dados: IDictionary) => {
          dataLabels.push(dados.NMLOCALCLIENTE);
          dataSeries.push(dados.QUANTIDADE);
          dataColors.push(dados.DSCORIDLOCAL);
        });
      } else {
        dataLabels.push('Nenhum dado disponível');
        dataSeries.push(100);
        dataColors.push('#57737a');
      }
    });
    const options = {colors: dataColors, series: dataSeries, labels: dataLabels};
    component.updateChart(options);
  }

  public latestReports({ component }:{ component:Grid }) {
    Http.get('/latestReports').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.reports;
    });
  }
}
